<?php
/***************************************************************************
 *   Copyright (C) 2005-2006 Artur Wiebe                                   *
 *   wibix@gmx.de                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
if(!defined("IN_FUSION")) {
	header("Location: ../../index.php");
	exit;
}
if(!iMEMBER) {
	return;
}


$do_send_pm = false;
unset($errno);


function awc_get_ind($coord)
{
	$row = 8 - (ord(substr($coord, 1, 1)) - ord('0'));
	$col = ord(substr($coord, 0, 1)) - ord('a');
	if($row<0 || $row>8 || $col<0 || $col>8) {
		return -1;
	}
	return $row*8+$col;
}

function awc_is_castling($from, $to, $board) {
	$p1 = $board[awc_get_ind($from)];
	$p2 = $board[awc_get_ind($to)];
	if(($p1=="k" && $p2=="t") || ($p1=="t" && $p2=="k")
		|| ($p1=="K" && $p2=="T") || ($p1=="T" && $p2=="K")) {
		return true;
	}
}


function awc_castling_move($from, $to, &$board) {
	$p1 = $board[awc_get_ind($from)];
	$p2 = $board[awc_get_ind($to)];

	if($p1=="k" || $p1=="K") {
		$king_piece = $p1;
		$tower_piece = $p2;
		$king = $from;
		$tower = $to;
	} else {
		$king_piece = $p2;
		$tower_piece = $p1;
		$king = $to;
		$tower = $from;
	}

	$king_index = awc_get_ind($king);
	$tower_index = awc_get_ind($tower);

/*
	echo "<p>K:".$king." (ind:".$king_index;
	echo "<p>T:".$tower." (ind:".$tower_index;
*/

	if($king_index < $tower_index) {
		$new_king_index = $king_index + 2;
		$new_tower_index = $king_index + 1;
	} else {
		$new_king_index = $king_index - 2;
		$new_tower_index = $king_index - 1;
	}

/*
	echo "<p>new_K_ind:".$new_king_index;
	echo "<p>new_T_ind:".$new_tower_index;
*/
	$board[$new_king_index] = $king_piece;
	$board[$new_tower_index] = $tower_piece;
	$board[awc_get_ind($from)] = "-";
	$board[awc_get_ind($to)] = "-";

	return "$king-X-$tower";
}

function awc_castling_move_undo($from, $to, $board) {
	$king_to = awc_get_ind($from);
	$king_from = awc_get_ind($from) - 2;
	$king_piece = $board[$king_from];
	$board = substr_replace($board, $king_piece, $king_to, 1);
	$board = substr_replace($board, "-", $king_from, 1);
/*
	echo "<p>$king_piece -> ".$king_to;
	echo "<p>- -> ".$king_from;
*/

	$tower_to = awc_get_ind($to);
	if($king_to < $tower_to) {
		$tower_from = $king_to + 1;
	} else {
		$tower_from = $king_to - 1;
	}
	$tower_piece = $board[$tower_from];
	$board = substr_replace($board, $tower_piece, $tower_to, 1);
	$board = substr_replace($board, "-", $tower_from, 1);
/*
	echo "<p>$tower_piece -> ".$tower_to;
	echo "<p>- -> ".$tower_from;
*/

	return $board;
}


// return 0 for no errors, error code else
function process_move($board_id, $move, $old_board, $old_moves, $white_next,
	$my_id) {
	global $locale;

	$my_cond = ($white_next
		? "white_next='Y' AND white_id=$my_id"
		: "white_next='N' AND black_id=$my_id");

	$from = $to = "";
	if(strlen($move)==5) {
		$from = substr($move, 0, 2);
		$to = substr($move, 3, 2);
	}
	if(awc_get_ind($from)<0 || awc_get_ind($to)<0) {
		return AWC_EMOVE;
	}

	// castling move
	if(awc_is_castling($from, $to, $old_board)) {
		$new_board = $old_board;
		$move = awc_castling_move($from, $to, $new_board);

	// "normal" move
	} else {
		$to_index = awc_get_ind($to);
		$from_index = awc_get_ind($from);
		$new_board = $old_board;

		$to_peace = $new_board[$to_index];
		$from_peace = $new_board[$from_index];

		if($to_peace!="-" &&	// empty field is ok.
			((ord($from_peace)>=97 && ord($to_peace)>=97)
			|| (ord($from_peace)<97 && ord($to_peace)<97))){
			return AWC_ECAPTURE;
		}
		if($to_peace=="k" || $to_peace=="K") {
			return AWC_EKINGKILL;
		}

		$new_board[$to_index] = $new_board[$from_index];
		$new_board[$from_index] = '-';

		if($to_peace != "-") {
			$move = str_replace("-", "-".$to_peace."-",
				$move);
		}
	}

	// update board.
	$query_id = dbquery("UPDATE ".DB_PREFIX."aw_chess"
		." SET board='$new_board',"
		." moves=CONCAT(moves, '".AW_SEP."', '$move'),"
		." mtime='".time()."',"
		." white_next='".($white_next ? "N" : "Y")."'"
		." WHERE id='$board_id' AND $my_cond");
	if(!$query_id) {
		return false;
	}
	if(mysql_affected_rows()==1) {
		return 0;	// no errors
	}
	return AWC_EUPDATE;
}



/*
 * action
 */
if(isset($_POST['gameover'])) {
	$my_id = $userdata['user_id'];
	$msg = trim(stripinput($_POST['msg']));
	if(empty($msg)) {
		fallback(FUSION_SELF."?id=$id&errno=".AWC_EREQUEST);
	}

	$query_id = dbquery("UPDATE ".DB_PREFIX."aw_chess"
		." SET state='".CHESS_REQUEST."', request_msg='$msg',"
		." request_id='$my_id'"
		." WHERE id='$id' AND state='".AW_RUNNING."'"
		." AND (black_id='$my_id' || white_id='$my_id')");
	if(mysql_affected_rows()!=1) {
		fallback(FUSION_SELF."?id=$id&errno=".AWC_EUPDATE);
	}

	awc_send_pm($my_id, AW_ADMIN_PM, $locale['AWC400'],
		"[url=".$settings['siteurl']
			."infusions/aw_chess/viewgame.php?id=$id"
			."]".$locale['AWC020']."[/url]");
	fallback(FUSION_SELF."?id=$id");

} elseif(isset($_GET['undo']) && isset($_POST['yes'])) {
	$user_id = $userdata['user_id'];
	$query_id = dbquery("SELECT board, moves, white_next"
		." FROM ".DB_PREFIX."aw_chess"
		." WHERE id='$id' AND moves!=''"
		." AND SUBSTRING(moves, -3, 1)!='".AW_UNDO."'"
		." AND (white_id='$user_id' OR black_id='$user_id')"
		." AND ((white_next='Y' AND white_id='$user_id')"
			." || (white_next='N' AND black_id='$user_id'))"
		);
	echo "<p>#".dbrows($query_id);
	if(dbrows($query_id)==1) {
		$data = dbarray($query_id);
		echo "<p><pre>".$data['board']."</pre>";
		echo "<p><pre>".$data['moves']."</pre>";

		$moves = explode(AW_SEP, $data['moves']);
		$count = count($moves);
		$last_move = $moves[$count - 1];

		echo "<p>Count_Moves:".$count."  [--]  ".$moves[$count-1];

		//
		$src = substr($last_move, 0, 2);
		$dst = substr($last_move, -2);
		$dst_index = awc_get_ind($dst);
		$src_index = awc_get_ind($src);

		if(strlen($last_move)==5) {
			$restore = "-";
			$dst_item = substr($data['board'], $dst_index, 1);
			$src_item = $restore;
		} elseif(strlen($last_move)==7) {
			$restore = "x";
		} else {
			$restore = substr($last_move, 3, 2);
			$dst_item = subbstr($data['board'], $dst_index, 1);
			$src_item = $restore;
		}

		//
		echo "<hr>";
		echo "<p>SRC:".$src;
		echo "<p>DST:".$dst;
		echo "<p>RPL:".$restore;


		if($restore!="x") {
			// src=dst
			$board_str = substr_replace($data['board'], $dst_item,
				$src_index, 1);
			// dst=restore
			$board_str = substr_replace($board_str, $restore,
				$dst_index, 1);
		// castling undo
		} else {
			$board_str = awc_castling_move_undo($src,
				$dst, $data['board']);
		}

		$moves_str = $data['moves']
			.AW_SEP.$dst.AW_UNDO.$restore.AW_UNDO.$src;

		//
		echo "<hr>";
		echo "<p><pre>$board_str</pre>";
		echo "<p><pre>$moves_str</pre>";

		$ok = dbquery("UPDATE ".DB_PREFIX."aw_chess"
			." SET board='$board_str', moves='$moves_str',"
			." mtime='".time()."',"
			." white_next='".($data['white_next']=="N" ? "Y" : "N")."'"
			." WHERE id='$id'");
		echo "<p><b>Q:</b><code>$query</code>";

		$do_send_pm = true;
	}

} elseif(isset($_POST['aw_do_move'])) {
	$errno = process_move($_POST['id'], $_POST['aw_move'],
		$_POST['aw_old_board'], $_POST['aw_old_moves'],
		$_POST['white_next']=="Y", $userdata['user_id']);
	if(!$errno) {
		$do_send_pm = true;
	}

} elseif(isset($_POST['save'])
	&& (iSUPERADMIN
		|| $user_id==$game['white_id'] || $user_id==$game['black_id'])){

	$msg = trim(stripinput($_POST['comment']));
	if(empty($msg)) {
		fallback(FUSION_SELF."?id=$id&errno=".AWC_ECOMMENT);
	}
	unset($commid);
	if(isset($_POST['commid'])) {
		if(isNum($_POST['commid']) && iSUPERADMIN) {// FIXME
			$commid = $_POST['commid'];
		} else {
			fallback(FUSION_SELF."?id=$id");// FIXME
		}
	}

	if(isset($commid)) {
		$ok = dbquery("UPDATE ".DB_PREFIX."aw_chess_comments"
			." SET message='$msg'"
			." WHERE id='$commid' AND game_id='$id'");
	} else {
		$ok = dbquery("INSERT INTO ".DB_PREFIX."aw_chess_comments"
			." (game_id, user_id, datestamp, message)"
			." VALUES"
			." ('$id', '$user_id', '".time()."', '$msg')");
		$commid = mysql_insert_id();
	}
	fallback(FUSION_SELF."?id=$id#comm$commid");

} elseif(isset($_GET['delcomm']) && isNum($_GET['delcomm']) && iSUPERADMIN) {
	$ok = dbquery("DELETE FROM ".DB_PREFIX."aw_chess_comments"
		." WHERE id='".$_GET['delcomm']."' AND game_id='$id'");
	fallback(FUSION_SELF."?id=$id");
}


if($do_send_pm) {
	$data = dbarray(dbquery("SELECT white_id, black_id"
		." FROM ".DB_PREFIX."aw_chess"
		." WHERE id='".$_POST['id']."'"));
	$opp_id = ($data['white_id']==$userdata['user_id']
		? $data['black_id']
		: $data['white_id']);
	awc_send_pm($userdata['user_id'], $opp_id, $locale['AWC401'],
		"[url=".$settings['siteurl']
			."infusions/aw_chess/viewgame.php?id=$id"
			."]".$locale['AWC020']."[/url]");
	fallback(FUSION_SELF."?id=$id&errno=$errno");

} elseif(isset($errno)) {
	fallback(FUSION_SELF."?id=$id&errno=$errno");
}


?>
